@@ -79,8 +79,8 @@ except ImportError as error1:
 
 __Conventions__  = 'CF-1.7'
 __author__       = 'David Hassell'
-__date__         = '2019-10-01'
-__version__      = '3.0.1'
+__date__         = '2019-10-31'
+__version__      = '3.0.2'
 __cf_version__   = cfdm.core.__cf_version__
 
 from distutils.version import LooseVersion
@@ -181,8 +181,6 @@ from .units                import Units
 
 from .fieldlist            import FieldList
 
-from .abstract             import Coordinate
-
 from .dimensioncoordinate  import DimensionCoordinate
 from .auxiliarycoordinate  import AuxiliaryCoordinate
 from .coordinatereference  import CoordinateReference
@@ -208,7 +206,7 @@ from .query                import (Query, lt, le, gt, ge, eq, ne, contain, conta
                                    cellwi, cellwo, djf, mam, jja, son, seasons)
 from .constants            import *
 from .functions            import *
-from .maths                import relative_vorticity
+from .maths                import relative_vorticity, histogram
 
 
 from .read_write import (read,
