@@ -8,9 +8,8 @@ from numpy.ma import array      as numpy_ma_array
 from numpy.ma import is_masked  as numpy_ma_is_masked
 from numpy.ma import masked_all as numpy_ma_masked_all
 
-from ..functions import parse_indices, get_subspace #, abspath
-from ..functions import inspect as cf_inspect
-from ..constants import CONSTANTS
+from ..functions import parse_indices, get_subspace
+from ..constants import masked as cf_masked
 
 _debug = False
 
@@ -35,7 +34,7 @@ class FilledArray(abstract.Array):
 
     fill_value : scalar, optional
 
-    masked_all: `bool`
+#    masked_all: `bool`
 
     '''
     def __init__(self, dtype=None, ndim=None, shape=None, size=None,
@@ -43,14 +42,13 @@ class FilledArray(abstract.Array):
         '''
         '''
         super().__init__(dtype=dtype, ndim=ndim, shape=shape,
-                         size=size, fill_value=fill_value,
-                         masked_all=masked_all)
-    #--- End: def
+                         size=size, fill_value=fill_value)
+
 
     def __getitem__(self, indices):
         '''x.__getitem__(indices) <==> x[indices]
 
-Returns a numpy array.
+    Returns a numpy array.
 
         '''
         if indices is Ellipsis:
@@ -79,37 +77,17 @@ Returns a numpy array.
                         array_shape.append(a)
                 else:
                     array_shape.append(len(index))
-            #-- End: for
         #-- End: if
 
-        if self.masked_all():
+        if self.fill_value() is cf_masked:
             return numpy_ma_masked_all(array_shape, dtype=self.dtype)
-        elif self.fill_value is not None:
+        elif self.fill_value() is not None:
             return numpy_full(array_shape, fill_value=self.fill_value(),
                               dtype=self.dtype)
         else:
             return numpy_empty(array_shape, dtype=self.dtype)
-    #--- End: def
-
-#    def __repr__(self):
-#        '''
-#
-#x.__repr__() <==> repr(x)
-#
-#'''
-#        return "<CF {0}: shape={1}, dtype={2}, fill_value={3}>".format(
-#            self.__class__.__name__, self.shape, self.dtype, self.fill_value)
-#    #--- End: def
-#
-#    def __str__(self):
-#        '''
-#
-#x.__str__() <==> str(x)
-#
-#'''
-#        return repr(self)
-#    #--- End: def
 
+        
     # ----------------------------------------------------------------
     # Attributes
     # ----------------------------------------------------------------
@@ -117,144 +95,145 @@ Returns a numpy array.
     def dtype(self):
         '''Data-type of the data elements.
          
-**Examples:**
-
->>> a.dtype
-dtype('float64')
->>> print(type(a.dtype))
-<type 'numpy.dtype'>
+    **Examples:**
+    
+    >>> a.dtype
+    dtype('float64')
+    >>> print(type(a.dtype))
+    <type 'numpy.dtype'>
 
         '''
         return self._get_component('dtype')
-    #--- End: def
+
     
     @property
     def ndim(self):
         '''Number of array dimensions
         
-**Examples:**
-
->>> a.shape
-(73, 96)
->>> a.ndim
-2
->>> a.size
-7008
-
->>> a.shape
-(1, 1, 1)
->>> a.ndim
-3
->>> a.size
-1
-
->>> a.shape
-()
->>> a.ndim
-0
->>> a.size
-1
+    **Examples:**
+    
+    >>> a.shape
+    (73, 96)
+    >>> a.ndim
+    2
+    >>> a.size
+    7008
+    
+    >>> a.shape
+    (1, 1, 1)
+    >>> a.ndim
+    3
+    >>> a.size
+    1
+    
+    >>> a.shape
+    ()
+    >>> a.ndim
+    0
+    >>> a.size
+    1
         '''
         return self._get_component('ndim')
-    #--- End: def
+
     
     @property
     def shape(self):
         '''Tuple of array dimension sizes.
 
-**Examples:**
-
->>> a.shape
-(73, 96)
->>> a.ndim
-2
->>> a.size
-7008
-
->>> a.shape
-(1, 1, 1)
->>> a.ndim
-3
->>> a.size
-1
-
->>> a.shape
-()
->>> a.ndim
-0
->>> a.size
-1
+    **Examples:**
+    
+    >>> a.shape
+    (73, 96)
+    >>> a.ndim
+    2
+    >>> a.size
+    7008
+    
+    >>> a.shape
+    (1, 1, 1)
+    >>> a.ndim
+    3
+    >>> a.size
+    1
+    
+    >>> a.shape
+    ()
+    >>> a.ndim
+    0
+    >>> a.size
+    1
         '''
         return self._get_component('shape')
-    #--- End: def
+
     
     @property
     def size(self):        
         '''Number of elements in the array.
 
-**Examples:**
-
->>> a.shape
-(73, 96)
->>> a.size
-7008
->>> a.ndim
-2
-
->>> a.shape
-(1, 1, 1)
->>> a.ndim
-3
->>> a.size
-1
-
->>> a.shape
-()
->>> a.ndim
-0
->>> a.size
-1
+    **Examples:**
+    
+    >>> a.shape
+    (73, 96)
+    >>> a.size
+    7008
+    >>> a.ndim
+    2
+    
+    >>> a.shape
+    (1, 1, 1)
+    >>> a.ndim
+    3
+    >>> a.size
+    1
+    
+    >>> a.shape
+    ()
+    >>> a.ndim
+    0
+    >>> a.size
+    1
 
         '''
         return self._get_component('size')
-    #--- End: def
+
 
     def fill_value(self):        
         '''TODO        '''
         return self._get_component('fill_value')
-    #--- End: def
 
-    def masked_all(self):        
-        '''TODO        '''
-        return self._get_component('masked_all')
-    #--- End: def
+
+#    def masked_all(self):        
+#        '''TODO        '''
+#        return self._get_component('masked_all')
+
 
     @property
     def array(self):
         '''TODO
-'''
+        '''
         return self[...]
 
+    
     def reshape(self, newshape):
+        '''TODO
         '''
-'''
         new = self.copy()        
         new.shape = newshape
         new.ndim  = len(newshape)
         return new
-    #--- End: def
+
 
     def resize(self, newshape):
+        '''TODO
         '''
-'''
         self.shape = newshape
         self.ndim  = len(newshape)
-    #--- End: def
+
 
     def view(self):
+        '''TODO
         '''
-'''
         return self[...]
-    #--- End: def
+
 
 #--- End: class
