@@ -1,9 +1,8 @@
 from .functions import (_DEPRECATION_ERROR,
                         _DEPRECATION_ERROR_KWARGS,
                         _DEPRECATION_ERROR_METHOD,
-                        _DEPRECATION_ERROR_ATTRIBUTE,
                         _DEPRECATION_ERROR_DICT,
-                        _DEPRECATION_ERROR_SEQUENCE)
+                        )
 
 
 class FieldList(list):
