@@ -5,7 +5,7 @@ from numpy import where as numpy_where
 from numpy import sum as numpy_sum
 from numpy import finfo as numpy_finfo
 from .data.data import Data
-from .coordinate import DimensionCoordinate
+from .dimensioncoordinate import DimensionCoordinate
 from .functions import REGRID_LOGGING
 from . import _found_ESMF
 if _found_ESMF:
@@ -338,7 +338,7 @@ an associated mask.
                              staggerloc=staggerLocs)
             
             # Populate the grid centres
-            for d in xrange(0, ndim):
+            for d in range(0, ndim):
                 if ndim < 3:
                     gridCentre = grid.get_coords(d,
                                      staggerloc=ESMF.StaggerLoc.CENTER)
@@ -346,7 +346,7 @@ an associated mask.
                     gridCentre = grid.get_coords(d,
                                      staggerloc=ESMF.StaggerLoc.CENTER_VCENTER)
                 gridCentre[...] = coords[d].array.reshape(
-                    [shape[d] if x == d else 1 for x in xrange(0, ndim)])
+                    [shape[d] if x == d else 1 for x in range(0, ndim)])
                 #--- End: if
             #--- End: for
             
@@ -357,8 +357,14 @@ an associated mask.
                 else:
                     gridCorner = grid.coords[ESMF.StaggerLoc.CORNER_VFACE]
                 #--- End: if
-                for d in xrange(0, ndim):
-                    boundsD = coords[d].get_bounds(create=True).array
+                for d in range(0, ndim):
+#                    boundsD = coords[d].get_bounds(create=True).array
+                    boundsD = coords[d].get_bounds(None)
+                    if boundsD is None:
+                        boundsD = coords[d].create_bounds()
+
+                    boundsD = boundsD.array
+                    
                     if shape[d] > 1:
                         tmp = numpy_empty(shape[d] + 1)
                         tmp[0:-1] = boundsD[:, 0]
@@ -367,7 +373,7 @@ an associated mask.
                     #--- End: if
                     gridCorner[d][...] = boundsD.reshape(
                         [shape[d] + 1 if x == d else 1
-                         for x in xrange(0, ndim)])
+                         for x in range(0, ndim)])
                 #--- End: for
             #--- End: if
         #--- End: if
